/*
	Author: rbe
	
	Description:
	 Creates nicely formatted hints.
	 Note: use "<br />" and not "\n" to make newlines
	
	Parameter(s):
	 _this: array of [type, value, args...] or option, where
		type = image|img|i, or text|t
		value = text or image source
		args = array [key, value]
		 |
		 - size, scalar/decimal (e.g. 1.67)
		 - color, string (in hex, e.g. "#FF0000")
		 - font, string (Zeppelin32, ..33, ..33Italic, Bitstream, TahomaB, LucidaConsoleB)
		 - align, string (left, center, right)
		 - valign, string (top, middle, bottom)
		 - underline, true|1
		 - shadow, true|1

	 and
		option = string
		 |
		 - silent: use "hintSilent"
		 - sound: use "hint" (default)
		 - paragraph: insert 2 <br /> after each block
		 - newline: insert 1 <br /> after each block (default)
		 - inline: no auto. newlines/<br /> elements
		 
		 Options apply to everything that follows.
		
	Example:
	
	#	[
	#		"silent",
	#		["t", "Some title", ["size", 1.67], ["color", "#00FF66"]],
	#		["t", "...some message."]
	#	] call RUBE_hint;
		 
	Returns:
	 void
*/

private ["_m", "_newlines", "_silent", "_nl", "_t", "_v", "_args"];

_m = "";
_newlines = 1;
_silent = false;

_nl = {
	switch (_this) do
	{
		case 1: { _m = _m + "<br />"; };
		case 2: { _m = _m + "<br /><br />"; };
	};
};

{
	switch (typeName _x) do
	{
		// options
		case "STRING":
		{
			switch (_x) do
			{
				case "silent": { _silent = true; };
				case "sound": { _silent = false; };
				case "paragraph": { _newlines = 2; };
				case "newline": { _newlines = 1; };
				case "inline": { _newlines = 0; };
			};
		};
		// lines
		case "ARRAY":
		{
			_t = _x select 0;
			_v = _x select 1;
			_args = "";

			for "_i" from 2 to ((count _x) - 1) do
			{
				_args = _args + format["%1='%2' ", ((_x select _i) select 0), ((_x select _i) select 1)];
			};
						
			switch (true) do
			{
				case (_t in ["image", "img", "i"]):
				{
					_m = _m + format["<img image='%1' %2 />", _v, _args];
					_newlines call _nl;
				};
				case (_t in ["text", "t"]):
				{
					_m = _m + format["<t %2>%1</t>", _v, _args];
					_newlines call _nl;
				};
			};
		};
	};
} forEach _this;

if (_silent) then
{
	hintSilent parseText _m;
} else
{
	hint parseText _m;
};
